@@ -219,7 +219,7 @@ gdm_local_display_factory_create_transient_display (GdmLocalDisplayFactory *fact
 #endif
 
         g_object_set (display,
-                      "seat-id", "seat0"
+                      "seat-id", "seat0",
                       "allow-timed-login", FALSE,
                       NULL);
 
@@ -450,7 +450,7 @@ gdm_local_display_factory_sync_seats (GdmLocalDisplayFactory *factory)
 #ifdef ENABLE_WAYLAND_SUPPORT
                         gboolean wayland_enabled = FALSE;
                         if (gdm_settings_direct_get_boolean (GDM_KEY_WAYLAND_ENABLE, &wayland_enabled)) {
-                                if (wayland_enabled) {
+                                if (wayland_enabled && g_file_test ("/usr/bin/Xwayland", G_FILE_TEST_IS_EXECUTABLE) ) {
                                         session_type = "wayland";
                                 }
                         }
@@ -678,7 +678,7 @@ register_factory (GdmLocalDisplayFactory *factory)
         if (factory->priv->connection == NULL) {
                 g_critical ("error getting system bus: %s", error->message);
                 g_error_free (error);
-                exit (1);
+                exit (EXIT_FAILURE);
         }
 
         factory->priv->skeleton = GDM_DBUS_LOCAL_DISPLAY_FACTORY (gdm_dbus_local_display_factory_skeleton_new ());
@@ -694,7 +694,7 @@ register_factory (GdmLocalDisplayFactory *factory)
                                                &error)) {
                 g_critical ("error exporting LocalDisplayFactory object: %s", error->message);
                 g_error_free (error);
-                exit (1);
+                exit (EXIT_FAILURE);
         }
 
         return TRUE;
