@@ -137,6 +137,10 @@ const char       *gdm_session_get_session_id              (GdmSession     *sessi
 gboolean          gdm_session_bypasses_xsession           (GdmSession     *session);
 GdmSessionDisplayMode gdm_session_get_display_mode  (GdmSession     *session);
 
+#ifdef ENABLE_WAYLAND_SUPPORT
+void              gdm_session_set_ignore_wayland          (GdmSession *session,
+                                                           gboolean    ignore_wayland);
+#endif
 gboolean          gdm_session_start_conversation          (GdmSession *session,
                                                            const char *service_name);
 void              gdm_session_stop_conversation           (GdmSession *session,
